@@ -8,6 +8,7 @@
 static const char key_Frequency[] = "Frequency=";
 static const char key_Channels[] = "Channels=";
 static const char key_BitsPerSample[] = "BitsPerSample=";
+static const char key_SampleEncoding[] = "SampleEncoding=";
 static const char key_MaxFrags[] = "MaxFrags=";
 static const char key_FragSize[] = "FragSize=";
 static const char key_BufSize[] = "BufSize=";
@@ -62,6 +63,8 @@ static const char key_StilOpen[] = "StilOpen=";  // case!
 
 static const char key_MainPosX[] = "MainPosX=";
 static const char key_MainPosY[] = "MainPosY=";
+static const char key_MainExtX[] = "MainExtX=";
+static const char key_MainExtY[] = "MainExtY=";
 static const char key_DirPosX[] = "DirPosX=";
 static const char key_DirPosY[] = "DirPosY=";
 static const char key_DirExtX[] = "DirExtX=";
@@ -147,7 +150,7 @@ void readMergedPanPosRight(voiceVol panLevels[4], const udword allPanPosRight)
 bool ConfigFile::load()
 {
     open(configPath);
-    while (status && !endOfFile())  // line-by-line loop
+    while (status && isGood && !endOfFile())  // line-by-line loop
     {
         readNextLine();
         // Skip blank and comment lines.
@@ -173,6 +176,10 @@ bool ConfigFile::load()
         {
             myAudioConfig.precision = atoi(retCurParseBuf());
         }
+        else if (isKey(key_SampleEncoding))
+        {
+            myAudioConfig.encoding = atoi(retCurParseBuf());
+        }
         else if (isKey(key_BufSize))
         {
             myAudioConfig.bufSize = atoi(retCurParseBuf());
@@ -297,6 +304,10 @@ bool ConfigFile::load()
             myConfig.geomMainDlg.x = atoi(retCurParseBuf());
         else if (isKey(key_MainPosY))
             myConfig.geomMainDlg.y = atoi(retCurParseBuf());
+        else if (isKey(key_MainExtX))
+            myConfig.geomMainDlg.w = atoi(retCurParseBuf());
+        else if (isKey(key_MainExtY))
+            myConfig.geomMainDlg.h = atoi(retCurParseBuf());
         // File dialog.
         else if (isKey(key_DirPosX))
             myConfig.geomDirDlg.x = atoi(retCurParseBuf());
@@ -320,9 +331,11 @@ bool ConfigFile::load()
         : (myEmuConfig.channels = SIDEMU_STEREO);
     (myAudioConfig.precision == 8) ? (myEmuConfig.bitsPerSample = SIDEMU_8BIT)
         : (myEmuConfig.bitsPerSample = SIDEMU_16BIT);
+    (myAudioConfig.encoding == 0x7f) ? (myEmuConfig.sampleFormat = SIDEMU_SIGNED_PCM)
+        : (myEmuConfig.sampleFormat = SIDEMU_UNSIGNED_PCM);
     
     close();
-    return status;
+    return isGood;
 }
 
 // ------------------------------------------------------------------- Output
@@ -392,6 +405,7 @@ bool ConfigFile::save()
             << key_Frequency << myAudioConfig.frequency << endl
             << key_Channels << myAudioConfig.channels << endl
             << key_BitsPerSample << myAudioConfig.precision << endl
+            << key_SampleEncoding << myAudioConfig.encoding << endl
             << key_BufSize << myAudioConfig.bufSize << endl
             << key_MaxFrags << myAudioConfig.maxFrags << endl
             << key_FragSize << myAudioConfig.fragSize << endl
@@ -461,6 +475,8 @@ bool ConfigFile::save()
         toFile
             << key_MainPosX << myConfig.geomMainDlg.x << endl
             << key_MainPosY << myConfig.geomMainDlg.y << endl
+            << key_MainExtX << myConfig.geomMainDlg.w << endl
+            << key_MainExtY << myConfig.geomMainDlg.h << endl
             << key_DirPosX << myConfig.geomDirDlg.x << endl
             << key_DirPosY << myConfig.geomDirDlg.y << endl
             << key_DirExtX << myConfig.geomDirDlg.w << endl
