@@ -29,15 +29,16 @@
 MainDialog::MainDialog(QWidget* parent,	const char* name) : Inherited(parent,name)
 {
     setCaption( "SIDPLAY" );
-    setFixedSize(width(),height());
+    setMinimumSize(width(),70);
     setIcon(*myMainIcon);
 
+    infoField->setFont(QFont("Courier",12,QFont::Normal,false,QFont::Latin1));
     infoField->setText("");
 	
     QPopupMenu *file = new QPopupMenu;
     CHECK_PTR( file );
     file->insertItem( "Open", this, SLOT(open()), CTRL+Key_O );
-    file->insertItem( "Save as", this, SLOT(saveAs()), CTRL+Key_O );
+    file->insertItem( "Save as", this, SLOT(saveAs()), CTRL+Key_S );
     file->insertSeparator();
     file->insertItem( "Exit", this, SLOT(quit()), CTRL+Key_Q );
 
@@ -46,7 +47,6 @@ MainDialog::MainDialog(QWidget* parent,	const char* name) : Inherited(parent,nam
     config->insertItem( "Audio", this, SLOT(openAudioDialog()) );
     config->insertItem( "Emulator", this, SLOT(openEmulatorDialog()) );
     config->insertItem( "SID Filter", this, SLOT(openFilterDialog()) );
-    config->insertItem( "Mixer", this, SLOT(openMixerDialog()) );
     config->insertItem( "HVSC Info", this, SLOT(openHVSCDialog()) );
 
     extra = new QPopupMenu;
@@ -54,6 +54,7 @@ MainDialog::MainDialog(QWidget* parent,	const char* name) : Inherited(parent,nam
     extra->setCheckable(true);
     stilID = extra->insertItem( "STIL View", this, SLOT(toggleStilDialog()) );
 	extra->insertItem( "History", this, SLOT(openHistoryDialog()) );
+    extra->insertItem( "Mixer", this, SLOT(openMixerDialog()) );
     waveViewID = extra->insertItem( "Oscilloscope", this, SLOT(openWaveViewDialog()) );
 
     QPopupMenu *help = new QPopupMenu;
@@ -70,6 +71,7 @@ MainDialog::MainDialog(QWidget* parent,	const char* name) : Inherited(parent,nam
     menu->insertItem( "&About", help );
 	
     myFileDlg = new QFileDialog(0,0,false);
+    myFileDlg->setMode(QFileDialog::ExistingFile);
     myFileDlg->setCaption("Sidtune Selection");
     myFileDlg->setIcon(*myMainIcon);
     connect(myFileDlg,SIGNAL(fileHighlighted(const char*)),this,SLOT(playFile(const char*)));
@@ -103,26 +105,43 @@ MainDialog::MainDialog(QWidget* parent,	const char* name) : Inherited(parent,nam
     connect( prevBtn, SIGNAL(released()), SLOT(prevSongReleased()) );
     prevBtn->setAutoRepeat( true );
     prevBtn->setAutoResize( false );
+    QToolTip::add(prevBtn,"Previous song");
 
     connect( stopBtn, SIGNAL(pressed()), SLOT(stopSongPressed()) );
     connect( stopBtn, SIGNAL(released()), SLOT(stopSongReleased()) );
     stopBtn->setAutoRepeat( false );
     stopBtn->setAutoResize( false );
+    QToolTip::add(stopBtn,"Stop playing");
 
     connect( playBtn, SIGNAL(pressed()), SLOT(playSongPressed()) );
     connect( playBtn, SIGNAL(released()), SLOT(playSongReleased()) );
     playBtn->setAutoRepeat( false );
     playBtn->setAutoResize( false );
+    QToolTip::add(playBtn,"Play/Pause");
 
     connect( ffBtn, SIGNAL(pressed()), SLOT(forwardSongPressed()) );
     connect( ffBtn, SIGNAL(released()), SLOT(forwardSongReleased()) );
     ffBtn->setAutoRepeat( false );
     ffBtn->setAutoResize( false );
+    QToolTip::add(ffBtn,"Fast forward");
 
     connect( nextBtn, SIGNAL(pressed()), SLOT(nextSongPressed()) );
     connect( nextBtn, SIGNAL(released()), SLOT(nextSongReleased()) );
     nextBtn->setAutoRepeat( true );
     nextBtn->setAutoResize( false );
+    QToolTip::add(nextBtn,"Next song");
+
+    subSongSlider = new QSlider(this,0);
+    subSongSlider->setOrientation(QSlider::Horizontal);
+    subSongSlider->setGeometry(140,60,136+64+12,10);
+    subSongSlider->setTracking(false);
+    subSongSlider->setValue(1);
+    subSongSlider->setRange(1,1);
+    subSongSlider->setSteps(1,1);
+    subSongSlider->setFocus();
+    connect(subSongSlider,SIGNAL(sliderMoved(int)),songLCD,SLOT(display(int)));
+    connect(subSongSlider,SIGNAL(valueChanged(int)),this,SLOT(newSubSong(int)));
+    QToolTip::add(subSongSlider,"Quick song selection");
 
     songLCD->display(1);
     QToolTip::add(songLCD,"Song number");
@@ -177,7 +196,8 @@ MainDialog::MainDialog(QWidget* parent,	const char* name) : Inherited(parent,nam
     myFilterDlg->setConfigDefault(myEmuConfig);  // *not* redundant
     myFilterDlg->setConfig(myEmuConfig);
     
-    hvscFileName = "";
+    // The currently active file name.
+    sidFileNameFull = sidFileNameHVSC = "";
 
     buffers = 2;
     multiBufferSize = 2048;
@@ -216,6 +236,17 @@ MainDialog::~MainDialog()
     delete next_lPic;
 }
 
+void MainDialog::resizeEvent(QResizeEvent*)
+{
+    int w = width();
+    int h = height();
+    
+    // rel. 365,195
+	sidInfoFrame->setGeometry(10,75,w-20,h-85);
+    infoField->setGeometry(15,80,w-30,h-95);
+}
+
+
 void MainDialog::enableWaveViewDlg(bool val)
 {
     menu->setItemEnabled(waveViewID,val);
@@ -281,24 +312,37 @@ void MainDialog::showSidTuneInfo()
 		mySidTune->getInfo(mySidTuneInfo);
 		
 		QString tmp;
-		tmp = "Name      : ";
-		tmp += mySidTuneInfo.infoString[0];
-		tmp += "\nAuthor    : ";
-		tmp += mySidTuneInfo.infoString[1];
-		tmp += "\nCopyright : ";
-		tmp += mySidTuneInfo.infoString[2];
+        if ( mySidTuneInfo.numberOfInfoStrings == 3 )
+        {
+            tmp = "Name      : ";
+            tmp += mySidTuneInfo.infoString[0];
+            tmp += "\nAuthor    : ";
+            tmp += mySidTuneInfo.infoString[1];
+            tmp += "\nCopyright : ";
+            tmp += mySidTuneInfo.infoString[2];
+        }
+        else
+        {
+            for ( int infoi = 0; infoi < mySidTuneInfo.numberOfInfoStrings; infoi++ )
+            {
+                if (infoi > 0)
+                    tmp += "\n";
+                tmp += "Info      : ";
+                tmp += mySidTuneInfo.infoString[infoi];
+            }
+        }
 		QString tmp2;
-		tmp2.sprintf("\nAddresses : $%04x, $%04x, $%04x",
-					 mySidTuneInfo.loadAddr,
-					 mySidTuneInfo.initAddr,
-					 mySidTuneInfo.playAddr);
-		tmp += tmp2;
 		tmp2.sprintf("\nSongs     : %d (Startsong: %d)",
 					 mySidTuneInfo.songs,
 					 mySidTuneInfo.startSong);
 		tmp += tmp2;
 		tmp2.sprintf("\nSong speed: %s",mySidTuneInfo.speedString);
 		tmp += tmp2;
+		tmp2.sprintf("\nAddresses : $%04x, $%04x, $%04x",
+					 mySidTuneInfo.loadAddr,
+					 mySidTuneInfo.initAddr,
+					 mySidTuneInfo.playAddr);
+		tmp += tmp2;
 		tmp2.sprintf("\nFormat    : %s",mySidTuneInfo.formatString);
 		tmp += tmp2;
 		infoField->setText(tmp);
@@ -324,6 +368,22 @@ void MainDialog::clearSidTuneInfo()
     infoField->setText(tmp);
 }
 
+void MainDialog::commandLinePlay(const char* fileName, const int song)
+{
+    if (fileName == 0)
+        return;
+    if (strcmp(fileName,"-") != 0)
+    {
+        // Who would use '-' for something else than stdin?
+        QFile testFile(fileName);
+        if (!testFile.exists())
+            return;
+    }
+    isHistoryFile = true;  // don`t put in history
+    loadSidTune(fileName,song);
+    loadStilData(fileName);
+}
+
 void MainDialog::playFile(const char* fileName)
 {
     isHistoryFile = false;
@@ -344,6 +404,10 @@ void MainDialog::playHistoryFile(const char* fileName)
 		loadSidTune(fileName,0);
 		loadStilData(fileName);
 	}
+    else
+    {
+        ; // file no longer exists
+    }
 }
 
 void MainDialog::quit()
@@ -381,7 +445,9 @@ void MainDialog::saveAs()
 
 void MainDialog::about()
 {
-	QString verCat = "SIDPLAY/X11   Version 1.3\n\n";
+	QString verCat = "SIDPLAY/X11   Version ";
+    verCat += VERSION;
+    verCat += "\n\n";
 	verCat += "Music player and SID chip emulator ";
 	verCat += myEmuEngine->getVersionString();
 	verCat += "\nby Michael Schwendt <sidplay@geocities.com>\n\n";
@@ -590,6 +656,9 @@ void MainDialog::setHVSCconfig(const HVSCconfig& inConfig)
 void MainDialog::receiveHVSCconfig(const HVSCconfig& inConfig)
 {
     myStilDlg->setConfig(inConfig);
+    initStilViewClass();
+    if (!isStilInfoEnabled())
+        myStilDlg->hide();
 }
 
 void MainDialog::receiveMixerConfig(const MixerConfig& inConfig)
@@ -664,16 +733,22 @@ void MainDialog::setVariousConfig(const VariousConfig& inConfig)
     
     if (inConfig.enableSTILview)
     {
-        setStilInfoEnabled(initStil());
+        setStilInfoEnabled(initStilViewClass());
     }
 
     if (inConfig.geomMainDlg.good())
-        setGeometry(inConfig.geomMainDlg.x,inConfig.geomMainDlg.y,
-                    width(),height());
-
+    {
+        if (inConfig.geomMainDlg.w < 0)
+            setGeometry(inConfig.geomMainDlg.x,inConfig.geomMainDlg.y,
+                        width(),height());
+        else
+            setGeometry(inConfig.geomMainDlg.x,inConfig.geomMainDlg.y,
+                        inConfig.geomMainDlg.w,inConfig.geomMainDlg.h);
+    } 
     myFileDlg->setDir(inConfig.currentDir);
     QFileDialog* myBakFileDlg = myFileDlg;
     myFileDlg = new QFileDialog(myBakFileDlg->dirPath(),inConfig.nameFilter,0,0,false);
+    myFileDlg->setMode(QFileDialog::ExistingFile);
     myFileDlg->setCaption("Sidtune Selection");
     myFileDlg->setIcon(*myMainIcon);
     delete myBakFileDlg;
@@ -697,12 +772,14 @@ bool MainDialog::writeConfigFile()
     myConfigFile.setEmuConfig(myEmuConfig);
     myConfigFile.setHVSCconfig(myHVSCDlg->getConfig());
     myVarConfig.usage = usage+myEmuEngine->getSecondsTotal();
-    myVarConfig.enableSTILview = extra->isItemChecked(stilID);
+    myVarConfig.enableSTILview = isStilInfoEnabled();
     myVarConfig.currentDir = myFileDlg->dirPath();
     myVarConfig.nameFilter = myFileDlg->dir()->nameFilter();
     myVarConfig.showDirDlg = myFileDlg->isVisible();
     myVarConfig.geomMainDlg.x = x();
     myVarConfig.geomMainDlg.y = y();
+    myVarConfig.geomMainDlg.w = width();
+    myVarConfig.geomMainDlg.h = height();
     myVarConfig.geomDirDlg.x = myFileDlg->x();
     myVarConfig.geomDirDlg.y = myFileDlg->y();
     myVarConfig.geomDirDlg.w = myFileDlg->width();
@@ -725,6 +802,46 @@ bool MainDialog::writeConfigFile()
 
 // --------------------------------------------------------------------------
 
+void MainDialog::newSubSong(int newSong)
+{
+    if (mySidTune->getStatus())
+    {
+        playerState playbackStateOld = playbackState;
+        playbackState = IS_INTERRUPTED;
+
+        mySidTune->getInfo(mySidTuneInfo);
+		if (newSong>=1 && newSong<=mySidTuneInfo.songs)
+		{
+			sidEmuInitializeSong(*myEmuEngine,*mySidTune,newSong);
+
+            resetTimeDisplay();
+            
+			mySidTune->getInfo(mySidTuneInfo);
+			if ( !mySidTune->getStatus() )
+			{
+				playbackStateOld = (playbackState = IS_STOPPED);
+			}
+			showSidTuneInfo();
+			showStilInfo();
+		}
+		playbackState = playbackStateOld;
+	}
+	adjustCommandButtons();
+	setPrevBtnState();
+	setNextBtnState();
+}
+
+void MainDialog::updateSubSongSlider()
+{
+    if (mySidTune->getStatus())
+    {
+        mySidTune->getInfo(mySidTuneInfo);
+        subSongSlider->setValue(mySidTuneInfo.currentSong);
+        subSongSlider->setRange(1,mySidTuneInfo.songs);
+        subSongSlider->setSteps(1,1);
+    }
+}
+
 void MainDialog::prevSongPressed()
 {
 	if (mySidTune->getStatus())
@@ -740,6 +857,7 @@ void MainDialog::prevSongPressed()
 			sidEmuInitializeSong(*myEmuEngine,*mySidTune,mySidTuneInfo.currentSong-1);
             
             resetTimeDisplay();
+            updateSubSongSlider();
             
 			mySidTune->getInfo(mySidTuneInfo);
 			if ( !mySidTune->getStatus() )
@@ -788,6 +906,7 @@ void MainDialog::nextSongPressed()
 			sidEmuInitializeSong(*myEmuEngine,*mySidTune,mySidTuneInfo.currentSong+1);
 
             resetTimeDisplay();
+            updateSubSongSlider();
             
 			mySidTune->getInfo(mySidTuneInfo);
 			if ( !mySidTune->getStatus() )
@@ -909,53 +1028,66 @@ void MainDialog::forwardSongReleased()
 
 void MainDialog::loadStilData(const char* fileName)
 {
-	hvscFileName = fileName;
+    // If the input filename contains a part which is relative to
+    // the HVSC root, this will be that part.
+    sidFileNameHVSC = "";
     
-	if ( !extra->isItemChecked(stilID) )
-		return;
+    // STIL View not enabled?
+    if ( !isStilInfoEnabled() )
+        return;
 
-	QString fileNameStr = fileName;
-	// Filename is somewhere in HVSC directory?
-	if (fileNameStr.find(myHVSCDlg->getConfig().hvscRootPath.absPath()) == 0)
-	{
-		// Copy HVSC relative filename.
-		hvscFileName = fileName+myHVSCDlg->getConfig().hvscRootPath.absPath().length();
-		showStilInfo();  // if any
-	}
+    QString fileNameStr = fileName;
+    // Is filename anywhere in HVSC tree?
+    if (fileNameStr.find(myHVSCDlg->getConfig().hvscRootPath.absPath()) == 0)
+    {
+        // Create filename which is relative to the HVSC root.
+        sidFileNameHVSC = 
+            fileNameStr.right(fileNameStr.length()-
+                              myHVSCDlg->getConfig().hvscRootPath.absPath().length());
+        showStilInfo();  // if any
+    }
     else
-		myStilDlg->hide();
+    {
+        // Nothing to show. Hence hide a possibly open STIL View dialog.
+        myStilDlg->hide();
+    }
 }
 
 void MainDialog::showStilInfo()
 {
-	if ( !extra->isItemChecked(stilID) )
+    // STIL View not enabled or no HVSC-relative file active?
+	if ( !isStilInfoEnabled() || sidFileNameHVSC.isEmpty())
 		return;
 
-	uword song = mySidTuneInfo.currentSong;
+	int song = mySidTuneInfo.currentSong;
 	if (myHVSCDlg->getConfig().showForFile)
-		song = 0;
-	myStilDlg->clearLabels();
-    bool stilIsOkay = true;
+		song = 0;              // 0 means: file-global STIL entry
+    
+	myStilDlg->clearLabels();  // drop previous entry
+    
+    bool stilIsOkay = true;    // we want to avoid successive errors
     char* globalComment;
     char* stilEntry;
     char* bugEntry;
+    // Now retrieve the individual STIL entries.
     if (stilIsOkay)
     {
-        globalComment = mySTIL.getGlobalComment(hvscFileName);
+        globalComment = mySTIL.getGlobalComment(sidFileNameHVSC);
         stilIsOkay = !mySTIL.hasCriticalError();
     }
     if (stilIsOkay)
     {
-        stilEntry = mySTIL.getEntry(hvscFileName,song);
+        stilEntry = mySTIL.getEntry(sidFileNameHVSC,song);
         stilIsOkay = !mySTIL.hasCriticalError();
     }
     if (stilIsOkay)
     {
-        bugEntry = mySTIL.getBug(hvscFileName,song);
+        bugEntry = mySTIL.getBug(sidFileNameHVSC,song);
         stilIsOkay = !mySTIL.hasCriticalError();
     }
     if (stilIsOkay)
     {
+        // Show the STIL entry.
         myStilDlg->setLabels(globalComment,stilEntry,bugEntry);
     }
     else
@@ -965,56 +1097,49 @@ void MainDialog::showStilInfo()
 		error.setText(mySTIL.getErrorStr());
 		error.adjustSize();
 		error.show();
+        // Disable STIL View.
+        // Protect the user from any further critical errors.
 		setStilInfoEnabled(false);
     }
 }
 
 void MainDialog::toggleStilDialog()
 {
-	if ( !extra->isItemChecked(stilID) )
-	{
-		extra->setItemChecked(stilID,initStil());
-        if (!hvscFileName.isEmpty())  // try to pop up STIL for
-        {                             // current song file (if available)
-            loadStilData(hvscFileName);
-            showStilInfo();
-        }
-	}
-	else
-	{
-		extra->setItemChecked(stilID,false);
-		myStilDlg->hide();
-	}
+    if ( !isStilInfoEnabled() )
+    {
+        // Enable STIL View.
+        setStilInfoEnabled(initStilViewClass());
+        // Try to show STIL entry of currently active file.
+        loadStilData(sidFileNameFull);
+    }
+    else
+    {
+        // Disable STIL View.
+        setStilInfoEnabled(false);
+        // Hide a possibly open STIL View dialog.
+        myStilDlg->hide();
+    }
 }
 
-bool MainDialog::isStilInfoEnabled()
+bool MainDialog::initStilViewClass()
 {
-	return extra->isItemChecked(stilID);
-}
-
-void MainDialog::setStilInfoEnabled(bool val)
-{
-	extra->setItemChecked(stilID,val);
-}
-
-bool MainDialog::initStil()
-{
-    // Do existence check on STIL database file.
-	if ( !mySTIL.setBaseDir(myHVSCDlg->getConfig().hvscRootPath.absPath()) )
-	{
-		QMessageBox error;
-		error.setIcon(QMessageBox::Warning);
-		error.setText(mySTIL.getErrorStr());
-		error.adjustSize();
-		error.show();
-		setStilInfoEnabled(false);
-		return false;
-	}
-	// Let the STIL View class do the rest.
-	else
-	{
-		return true;
-	}
+    // Let STIL View class do check for STIL database file.
+    if ( !mySTIL.setBaseDir(myHVSCDlg->getConfig().hvscRootPath.absPath()) )
+    {
+        QMessageBox error;
+        error.setIcon(QMessageBox::Warning);
+        error.setText(mySTIL.getErrorStr());
+        error.adjustSize();
+        error.show();
+        // Disable STIL View.
+        setStilInfoEnabled(false);
+        return false;
+    }
+    // STIL View class initialized.
+    else
+    {
+        return true;
+    }
 }
 
 // ------------------------------------------------------------------- Player
@@ -1027,6 +1152,9 @@ void MainDialog::loadSidTune(const char* fileName, const int commandLineStartSon
 
 	bool loadSucceeded = false;
 	
+    // Make a copy for later use.
+    sidFileNameFull = fileName;
+    
 	// sidTune::open() does not support standard input.
 	if (strcmp(fileName,"-") == 0)
 	{
@@ -1062,6 +1190,8 @@ void MainDialog::loadSidTune(const char* fileName, const int commandLineStartSon
 		clearSidTuneInfo();
 		
 		infoField->setText(mySidTuneInfo.statusString);
+        // With too many errors one has to click too much.
+        //
 		//QMessageBox error;
 		//error.setIcon(QMessageBox::Warning);
 		//error.setText(mySidTuneInfo.statusString);
@@ -1076,6 +1206,7 @@ void MainDialog::loadSidTune(const char* fileName, const int commandLineStartSon
 				sidEmuInitializeSong(*myEmuEngine,*mySidTune,commandLineStartSong);
 			else
 				sidEmuInitializeSong(*myEmuEngine,*mySidTune,mySidTuneInfo.startSong);
+            updateSubSongSlider();
 			mySidTune->getInfo(mySidTuneInfo);
             if (!isHistoryFile)
             {
@@ -1116,6 +1247,8 @@ void MainDialog::timerJob()
             // impossible to determine the current playback position in an
             // accurate way. Normally, this would allow synchronization of
             // audio and graphics output.
+            //
+            // Let's don't care about this...
 			
 			sidEmuFillBuffer(*myEmuEngine,*mySidTune,pBuffer[currentBuffer],multiBufferSize);
 			myAudio.play(pBuffer[currentBuffer],multiBufferSize);
@@ -1212,6 +1345,7 @@ void MainDialog::startPlaying()
 		}
 		else
 		{
+            updateSubSongSlider();
 			setPrevBtnState();
 			setNextBtnState();
 			showSidTuneInfo();
