@@ -63,9 +63,11 @@ public:
     void enableWaveViewDlg(bool);
 
     void showStilInfo();
+    bool initStilViewClass();
+    // For config saver and internal use as wrapper.
     bool isStilInfoEnabled();
+    // For config loader and internal use as wrapper.
     void setStilInfoEnabled(bool);
-    bool initStil();
 
     bool writeConfigFile();
 
@@ -95,10 +97,9 @@ public slots:
     void receiveHVSCconfig(const HVSCconfig&);
     void receiveMixerConfig(const MixerConfig&);
     
+    void commandLinePlay(const char*, const int);
     void playFile(const char*);
     void playHistoryFile(const char*);
-    void loadStilData(const char*);
-    void loadSidTune(const char*, const int);
 	
     virtual void open();
     virtual void saveAs();
@@ -144,6 +145,9 @@ public slots:
     QPixmap* play_l_pPic;
     QPixmap* next_l_pPic;
 
+    void loadStilData(const char*);
+    void loadSidTune(const char*, const int);
+    
     bool openAudioDriver();
     void closeAudioDriver();
     
@@ -158,6 +162,8 @@ public slots:
     void openAudioBufferSystem();
     void freeAudioBufferSystem();
     
+    void resizeEvent(QResizeEvent*);
+    
  protected slots:
 
 	virtual void done(int);
@@ -172,9 +178,13 @@ public slots:
     void forwardSongReleased();
     void nextSongPressed();
     void nextSongReleased();
+    void newSubSong(int);
+    void updateSubSongSlider();
 
  private:
 	
+    QSlider* subSongSlider;
+
     QMenuBar *menu;
     QPopupMenu *extra;
     int waveViewID, stilID;
@@ -202,7 +212,9 @@ public slots:
     AudioDriver myAudio;
     
     STIL mySTIL;
-    QString hvscFileName;
+    // The currently active file name, and the part of it
+    // which is relative to the HVSC root (if any).
+    QString sidFileNameFull, sidFileNameHVSC;
 
     // Mostly unimplemented! Multi-buffering system.
     
@@ -221,4 +233,15 @@ public slots:
     
 };
 
+inline bool MainDialog::isStilInfoEnabled()
+{
+    return extra->isItemChecked(stilID);
+}
+
+inline void MainDialog::setStilInfoEnabled(bool val)
+{
+    extra->setItemChecked(stilID,val);
+}
+
+
 #endif  // MainDialog_h
