@@ -23,9 +23,9 @@
 #define STILopenFlags (ios::in | ios::binary | ios::nocreate)
 #endif
 
-#define VERSION_NO 1.23
+#define VERSION_NO 1.24
 
-#define CERR_STIL_DEBUG if (STIL_DEBUG) cerr << "Line #" << __LINE__ << "STIL::"
+#define CERR_STIL_DEBUG if (STIL_DEBUG) cerr << "Line #" << __LINE__ << " STIL::"
 
 // CONSTRUCTOR
 STIL::STIL()
@@ -400,6 +400,7 @@ STIL::getGlobalComment(const char *relPathToEntry)
     char dir[STIL_MAX_PATH_SIZE];
     size_t endOfPath;
     char *temp;
+    char *lastSlash;
 
     lastError = NO_STIL_ERROR;
 
@@ -407,7 +408,14 @@ STIL::getGlobalComment(const char *relPathToEntry)
 
     // Save the dirpath.
 
-    endOfPath = strrchr(relPathToEntry, '/')-relPathToEntry+1;
+    lastSlash = (char *)strrchr(relPathToEntry, '/');
+
+    if (lastSlash == NULL) {
+        lastError = WRONG_DIR;
+        return NULL;
+    }
+
+    endOfPath = lastSlash-relPathToEntry+1;
     strncpy(dir, relPathToEntry, endOfPath);
     *(dir+endOfPath) = '\0';
 
