@@ -42,17 +42,17 @@ int main(int argc, char** argv)
     unsigned int inFile = 0;
     int commandLineStartSong = 0;
         
-    for (int a = 1; a < argc; a++)  
+    for (int myArg = 1; myArg < argc; myArg++)  
     {
         // Check for options prefix "-".
-        if (argv[a][0] == '-')  
+        if (argv[myArg][0] == '-')  
         {
             // Single "-" means reading from stdin.
-            if (strlen(argv[a]) == 1)
+            if (strlen(argv[myArg]) == 1)
 			{
                 if ( inFile == 0 )
                 {
-                    inFile = a;
+                    inFile = myArg;
                     break;
                 }
                 else
@@ -62,16 +62,16 @@ int main(int argc, char** argv)
             }
             else
             {
-                if (strnicmp(&argv[a][1],"-expert",7) == 0)
+                if (strnicmp(&argv[myArg][1],"-expert",7) == 0)
                     expertMode = true;
-                else if (strnicmp(&argv[a][1],"bufsize",7) == 0)
-                    argLineAC.bufSize = atoi(&argv[a][1+7]);
-                else if (strnicmp(&argv[a][1],"maxfrags",8) == 0)
-                    argLineAC.maxFrags = atoi(&argv[a][1+8]);
-                else if (strnicmp(&argv[a][1],"fragsize",8) == 0)
-                    argLineAC.fragSize = atoi(&argv[a][1+8]);
-                else if (strnicmp(&argv[a][1],"o",1) == 0)
-                    commandLineStartSong = atoi(&argv[a][1+1]);
+                else if (strnicmp(&argv[myArg][1],"bufsize",7) == 0)
+                    argLineAC.bufSize = atoi(&argv[myArg][1+7]);
+                else if (strnicmp(&argv[myArg][1],"maxfrags",8) == 0)
+                    argLineAC.maxFrags = atoi(&argv[myArg][1+8]);
+                else if (strnicmp(&argv[myArg][1],"fragsize",8) == 0)
+                    argLineAC.fragSize = atoi(&argv[myArg][1+8]);
+                else if (strnicmp(&argv[myArg][1],"o",1) == 0)
+                    commandLineStartSong = atoi(&argv[myArg][1+1]);
             }
         }
         else
@@ -79,7 +79,7 @@ int main(int argc, char** argv)
             // No prefix. Thus is considered a filename argument.
             if ( inFile == 0 )
             {
-                inFile = a;
+                inFile = myArg;
             }
             else 
             {
@@ -118,7 +118,7 @@ int main(int argc, char** argv)
         if (argLineAC.fragSize != myAudioConfig.fragSize)
             myAudioConfig.fragSize = argLineAC.fragSize;
         else
-            myAudioConfig.fragSize = 11;
+            myAudioConfig.fragSize = 2048;
             
         if (myAudioConfig.bufSize < 4*280)  // waveView w=280!
             myAudioConfig.bufSize = 4*280;
@@ -136,7 +136,7 @@ int main(int argc, char** argv)
     if (inFile != 0)
     {
         //cout << "Loading sidtune ..." << endl;
-        myMainDlg->loadSidTune(argv[inFile],commandLineStartSong);
+        myMainDlg->commandLinePlay(argv[inFile],commandLineStartSong);
     }
 
     // Give control to Qt.
